/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "cubo.h"

Cubo::Cubo( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	hace_cubo( v1, 1.0 );
}


void Cubo::timerDone( )
{
	angle += 0.67*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );

}

void Cubo::hace_cubo( float v[][3], float lado )
{
	v[0][0] = 0.0; v[0][1] = 0.0; v[0][2] = 0.0;
	v[1][0] = 0.0; v[1][1] = 0.0; v[1][2] =lado;
	v[2][0] =lado; v[2][1] = 0.0; v[2][2] =lado;
	v[3][0] =lado; v[3][1] = 0.0; v[3][2] = 0.0;

	v[4][0] = 0.0; v[4][1] =lado; v[4][2] = 0.0;
	v[5][0] = 0.0; v[5][1] =lado; v[5][2] =lado;
	v[6][0] =lado; v[6][1] =lado; v[6][2] =lado;
	v[7][0] =lado; v[7][1] =lado; v[7][2] = 0.0;
}

void Cubo::dibuja_cubo( float v[][3] )
{
	int c, i, j;

	static float colores[6][3] = {
		{ 1.0, 1.0, 0.0 }, // amarillo
		{ 1.0, 1.0, 1.0 }, // blanco
		{ 1.0, 0.0, 0.0 }, // rojo
		{ 0.0, 1.0, 1.0 }, // magenta
		{ 0.0, 0.0, 1.0 }, // azul
		{ 1.0, 0.5, 0.0 }, // naranja
	};
	static int indices[6][4] = {
		{ 1, 2, 6, 5},
		{ 0, 4, 7, 3},
		{ 2, 3, 7, 6},
		{ 0, 1, 5, 4},
		{ 4, 5, 6, 7},
		{ 0, 3, 2, 1} };

	glBegin( GL_QUADS );
	for( c=0; c<6; c++ ) {
		glColor3fv( &colores[c][0] ); 
		for( i=0; i<4; i++ ) {
			j = indices[c][i];
			glVertex3fv( &v[j][0] );
		}
	}
	glEnd( );
}

void Cubo::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Cubo::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Cubo::resizeGL( int l, int a )
{
	float aspect=(float)l/(float)a;

	windowWidth = l;
	windowHeight = a;

	glViewport (0, 0, (GLsizei) l, (GLsizei) a);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	float x = 2.5;
	float y = 2.5;

	if( l <= a )
		glOrtho ( -x, x, -y/aspect, y/aspect, -10.0, 10.0);
	else
		glOrtho ( -x*aspect, x*aspect, -y, y, -10.0, 10.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	emit changeSize ( );
}

QSizePolicy Cubo::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Cubo::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix( );
	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);


	glRotatef (-45.0, 1.0, 0.0, 0.0);
	glRotatef (  35.26, 0.0, 1.0, 0.0);
	glRotatef ( angle , 0.0, 0.0, 1.0);
	glRotatef ( -35.26, 0.0, 1.0, 0.0);
	glRotatef ( 45.0, 1.0, 0.0, 0.0);

	dibuja_cubo( v1 );	

	glPopMatrix( );

	// swapBuffers( );
}

void Cubo::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */
}


